import 'dart:core';
import 'package:flutter/material.dart';
import 'package:tim_ui_kit_lbs_plugin/abstract/map_class.dart';
import 'package:tim_ui_kit_lbs_plugin/abstract/map_widget.dart';
import 'package:tim_ui_kit_lbs_plugin/utils/color.dart';
import 'package:tim_ui_kit_lbs_plugin/utils/location_utils.dart';
import 'package:tim_ui_kit_lbs_plugin/utils/tim_location_model.dart';
import 'package:tim_ui_kit_lbs_plugin/utils/utils.dart';
import 'package:tim_ui_kit_lbs_plugin/widget/location_input.dart';
import 'package:tim_ui_kit_lbs_plugin/widget/location_list_item.dart';

class LocationPicker extends StatefulWidget {
  /// 地理位置选择完成后的onChange事件，返回一个LocationMessage，可用于发送消息。
  /// 特别说明：由于腾讯云IM位置消息仅支持传递一个desc字符串，因此此处的LocationMessage.desc将名称及地址拼接传递，格式："腾讯大厦/////深圳市南山区深南大道10000号"。
  /// 该拼接格式可被本插件所需地方解析，请放心使用。
  final ValueChanged<LocationMessage> onChange;

  /// 传入根据选定地图SDK实例化后的LocationUtils
  final LocationUtils locationUtils;

  /// 用于还未加载出来定位时，打开页面后，默认的中心点。
  final TIMCoordinate? initCoordinate;

  /// 传入根据选定地图SDK实例化后的地图组件TIMMapWidget
  final TIMMapWidget Function(
      VoidCallback onMapLoadDone,
      Key mapKey,
      Function(TIMCoordinate? targetGeoPt,
              TIMRegionChangeReason regionChangeReason)
          onMapMoveEnd) mapBuilder;

  const LocationPicker(
      {Key? key,
      required this.onChange,
      required this.mapBuilder,
      required this.locationUtils,
        this.initCoordinate})
      : super(key: key);

  @override
  State<StatefulWidget> createState() => LocationPickerState();
}

class LocationPickerState extends State<LocationPicker> {
  GlobalKey<TIMMapState> mapKey = GlobalKey();
  List<TIMPoiInfo> poiInfoList = [];
  String currentSearchCity = "深圳市";
  TIMCoordinate? currentCoordinate;
  String currentSelectedPOI = "";
  String currentLocationName = "";
  TextEditingController inputKeywordEditingController = TextEditingController();
  String dividerForDesc = "/////";

  @override
  void initState() {
    super.initState();
  }

  void _onGetPoiCitySearchResult(
      List<TIMPoiInfo>? poiInfoListResult, bool isError) {
    if (isError) {
      var error = ("检索失败");
      Utils.toast(error);
      print(error);
      return;
    }

    if (poiInfoListResult != null && poiInfoListResult.isNotEmpty) {
      TIMPoiInfo firstPOI = poiInfoListResult[0];
      mapKey.currentState?.moveMapCenter(firstPOI.pt!);
      setState(() {
        currentSearchCity = firstPOI.city ?? "深圳市";
        currentCoordinate =
            TIMCoordinate(firstPOI.pt!.latitude, firstPOI.pt!.longitude);
        currentSelectedPOI = firstPOI.uid!;
        currentLocationName =
            "${firstPOI.name}$dividerForDesc${firstPOI.address}";
      });
    }

    setState(() {
      poiInfoList = poiInfoListResult ?? [];
    });
  }

  void _onGetReverseGeoCodeSearchResult(
      TIMReverseGeoCodeSearchResult result, bool isError) {
    if (isError) {
      var error = ("检索失败");
      Utils.toast(error);
      print(error);
      return;
    }
    inputKeywordEditingController.text = "";
    List<TIMPoiInfo> poiInfoListWithCurrent = [
      // 将实时定位点补入POI列表第一项作为默认位置
      TIMPoiInfo(
        name: result.sematicDescription,
        pt: result.location,
        uid: "0312",
        address: result.address,
        city: result.addressDetail?.city ?? "深圳市",
      ),
      ...?result.poiList
    ];
    setState(() {
      poiInfoList = poiInfoListWithCurrent;
      currentSearchCity = result.addressDetail?.city ?? "深圳市";
      currentCoordinate = result.location;
      currentSelectedPOI = "0312";
      currentLocationName =
          "${result.sematicDescription}$dividerForDesc${result.address}";
    });
  }

  List<Widget> renderPOIList() {
    return poiInfoList.map((item) {
      return LocationLstItem(
        name: item.name!,
        isSelected: item.uid == currentSelectedPOI,
        address: item.address,
        onClick: () {
          setState(() {
            currentSelectedPOI = item.uid!;
            currentSearchCity = item.city ?? currentSearchCity;
            currentCoordinate = item.pt;
            currentLocationName = "${item.name}$dividerForDesc${item.address}";
          });
          mapKey.currentState?.moveMapCenter(item.pt!);
        },
      );
    }).toList();
  }

  void _onSendMsg() {
    widget.onChange(LocationMessage(currentLocationName,
        currentCoordinate!.longitude, currentCoordinate!.latitude));
    Navigator.pop(context);
  }

  Future<void> moveToCurrentLocation() async {
    // 若使用百度等其他地图提供的定位能力，一般可以获得更精准的位置/返回周边POI列表，此时可解除这里的注释
    // widget.locationUtils.moveToCurrentLocationActionWithSearchPOI(
    //   onGetReverseGeoCodeSearchResult: _onGetReverseGeoCodeSearchResult,
    //   moveMapCenter: mapKey.currentState!.moveMapCenter,
    // );

    final TIMCoordinate? currentLocation = await widget.locationUtils
        .moveToCurrentLocationActionWithoutSearchPOI(
            moveMapCenter: mapKey.currentState!.moveMapCenter);

    if (currentLocation != null) {
      print(
          "current location ${currentLocation.latitude} ${currentLocation.longitude}");
      widget.locationUtils.searchPOIByCoordinate(
          coordinate: currentLocation,
          onGetReverseGeoCodeSearchResult: _onGetReverseGeoCodeSearchResult);
    }
  }

  void onMapLoadDone() {
    if(widget.initCoordinate != null){
      mapKey.currentState!.moveMapCenter(widget.initCoordinate!);
    }
    moveToCurrentLocation();
  }

  void onMapMoveEnd(
      TIMCoordinate? targetGeoPt, TIMRegionChangeReason regionChangeReason) {
    // 如果是手动拖动地图到了新位置，将中心点的坐标重新搜索
    if (targetGeoPt != null &&
        regionChangeReason == TIMRegionChangeReason.Gesture) {
      widget.locationUtils.searchPOIByCoordinate(
          coordinate: targetGeoPt,
          onGetReverseGeoCodeSearchResult: _onGetReverseGeoCodeSearchResult);
      inputKeywordEditingController.text = "";
    }
  }

  @override
  Widget build(BuildContext context) {
    final debouncePOICitySearch = LocationUtils.debounce((keyword) {
      widget.locationUtils.poiCitySearch(
          onGetPoiCitySearchResult: _onGetPoiCitySearchResult,
          keyword: keyword,
          city: currentSearchCity);
    });
    return Scaffold(
      body: Container(
        decoration: const BoxDecoration(color: Colors.white),
        child: GestureDetector(
          onTap: () {
            FocusScopeNode currentFocus = FocusScope.of(context);
            if (!currentFocus.hasPrimaryFocus) {
              currentFocus.unfocus();
            }
          },
          child: Column(
            mainAxisAlignment: MainAxisAlignment.start,
            children: [
              Expanded(
                child: Stack(
                  alignment: AlignmentDirectional.center,
                  children: [
                    widget.mapBuilder(
                      onMapLoadDone,
                      mapKey,
                      onMapMoveEnd,
                    ),
                    Positioned(
                        child: Container(
                      margin: const EdgeInsets.only(bottom: 22),
                      child: const Icon(
                        Icons.place,
                        color: CommonColor.primaryColor,
                        size: 44,
                      ),
                    )),
                    Positioned(
                      right: 16,
                      bottom: 26,
                      child: GestureDetector(
                        onTap: moveToCurrentLocation,
                        child: Container(
                          decoration: const BoxDecoration(
                            color: Colors.white,
                            borderRadius: BorderRadius.all(Radius.circular(22)),
                            boxShadow: [
                              BoxShadow(
                                blurRadius: 6,
                                spreadRadius: 0.4,
                                color: CommonColor.weakTextColor, //阴影颜色
                              ),
                            ],
                          ),
                          child: const SizedBox(
                            child: Icon(
                              Icons.my_location,
                              color: CommonColor.primaryColor,
                              size: 24,
                            ),
                            height: 44,
                            width: 44,
                          ),
                        ),
                      ),
                    ),
                    Positioned(
                      top: 0,
                      left: 0,
                      child: Container(
                        constraints: BoxConstraints(
                            minWidth: MediaQuery.of(context).size.width,
                            minHeight: 100),
                        decoration: const BoxDecoration(
                          gradient: LinearGradient(
                            begin: Alignment.topCenter,
                            end: Alignment.bottomCenter,
                            colors: [
                              Colors.black38,
                              Colors.black38,
                              Color.fromRGBO(0, 0, 0, 0),
                            ],
                            stops: [0, 0.67, 1],
                          ),
                        ),
                      ),
                    ),
                    Positioned(
                      top: 52,
                      left: 28,
                      child: GestureDetector(
                        onTap: () => Navigator.pop(context),
                        child: const Text(
                          "取消",
                          style: TextStyle(color: Colors.white, fontSize: 16),
                        ),
                      ),
                    ),
                    Positioned(
                        top: 48,
                        right: 28,
                        child: ElevatedButton(
                          style: ButtonStyle(
                            backgroundColor: MaterialStateProperty.all(
                                CommonColor.primaryColor),
                          ),
                          onPressed: _onSendMsg,
                          child: const Text(("发送")),
                        )),
                  ],
                ),
              ),
              LocationInput(
                onChange: (String keyword) {
                  debouncePOICitySearch(keyword);
                },
                controller: inputKeywordEditingController,
              ),
              Container(
                constraints: BoxConstraints(
                  maxHeight: MediaQuery.of(context).size.height / 10 * 3,
                  maxWidth: MediaQuery.of(context).size.width,
                ),
                padding: const EdgeInsets.fromLTRB(12, 0, 16, 0),
                decoration: const BoxDecoration(
                  color: Colors.white,
                ),
                child: SingleChildScrollView(
                  child: Column(
                    children: renderPOIList(),
                  ),
                ),
              )
            ],
          ),
        ),
      ),
    );
  }
}
