import 'package:flutter/cupertino.dart';

abstract class TIMLocationBaseModel {
  /// model -> map
  @required
  Map<String, Object?> toMap();

  /// map -> dynamic
  @required
  dynamic fromMap(Map map);
}

class LocationMessage {
  final String desc;
  final double longitude;
  final double latitude;

  LocationMessage(this.desc, this.longitude, this.latitude);
}

/// 代表经纬度
class TIMCoordinate implements TIMLocationBaseModel {
  /// 纬度
  late double latitude;

  /// 经度
  late double longitude;

  /// TIMCoordinate构造方法
  TIMCoordinate(this.latitude, this.longitude);

  /// map => TIMCoordinate
  TIMCoordinate.fromMap(Map map)
      : assert(map.containsKey('latitude'),
  'Construct a TIMCoordinate，The parameter latitude cannot be null'),
        assert(map.containsKey('longitude'),
        'Construct a TIMCoordinate，The parameter longitude cannot be null') {
    latitude = map['latitude'] as double;
    longitude = map['longitude'] as double;
  }

  @override
  Map<String, Object> toMap() {
    return {'latitude': latitude, 'longitude': longitude};
  }

  @override
  fromMap(Map map) {
    return TIMCoordinate.fromMap(map);
  }
}

/// POI信息类
class TIMPoiInfo implements TIMLocationBaseModel {
  /// POI名称
  String? name;

  /// POI坐标
  TIMCoordinate? pt;

  /// POI地址信息
  String? address;

  /// POI唯一标识符uid
  String? uid;

  /// POI所在省份
  String? province;

  /// POI所在城市
  String? city;

  /// TIMPoiInfo构造方法
  TIMPoiInfo({
    this.name,
    this.pt,
    this.address,
    this.uid,
    this.province,
    this.city,
  });

  /// map => TIMPoiInfo
  TIMPoiInfo.fromMap(Map map)
      : assert(
  map != null, // ignore: unnecessary_null_comparison
  'Construct a TIMPoiInfo，The parameter map cannot be null !') {
    name = map['name'];
    pt = map['pt'] == null ? null : TIMCoordinate.fromMap(map['pt']);
    address = map['address'];
    uid = map['uid'];
    province = map['province'];
    city = map['city'];
  }
  @override
  fromMap(Map map) {
    return TIMPoiInfo.fromMap(map);
  }

  @override
  Map<String, Object?> toMap() {
    return {
      'name': name,
      'pt': pt?.toMap(),
      'address': address,
      'uid': uid,
      'province': province,
      'city': city,
    };
  }
}

/// 根据地理坐标反向查询结果类
class TIMReverseGeoCodeSearchResult implements TIMLocationBaseModel {
  /// 地址坐标
  TIMCoordinate? location;

  /// 地址名称
  String? address;

  /// 层次化地址信息
  TIMAddressComponent? addressDetail;

  /// 地址周边POI信息，成员类型为BMKPoiInfo
  List<TIMPoiInfo>? poiList;

  /// 结合当前位置POI的语义化结果描述
  String? sematicDescription;

  /// 有参构造
  TIMReverseGeoCodeSearchResult({
    this.location,
        this.address,
        this.addressDetail,
        this.poiList,
        this.sematicDescription,
    });

  /// map => TIMReverseGeoCodeSearchResult
  TIMReverseGeoCodeSearchResult.fromMap(Map map)
      : assert(
  map != null, // ignore: unnecessary_null_comparison
  'Construct a TIMReverseGeoCodeSearchResult，The parameter map cannot be null !') {
    location =
    map['location'] == null ? null : TIMCoordinate.fromMap(map['location']);
    address = map['address'];
    addressDetail = map['addressDetail'] == null
        ? null
        : TIMAddressComponent.fromMap(map['addressDetail']);
    if (map['poiList'] != null) {
      List<TIMPoiInfo> tmpPoiList = [];
      map['poiList'].forEach((v) {
        tmpPoiList.add(TIMPoiInfo.fromMap(v as Map));
      });
      poiList = tmpPoiList;
    }
    sematicDescription = map['sematicDescription'];
  }

  @override
  fromMap(Map map) {
    return TIMReverseGeoCodeSearchResult.fromMap(map);
  }

  @override
  Map<String, Object?> toMap() {
    return {
      'location': location?.toMap(),
      'address': address,
      'addressDetail': addressDetail?.toMap(),
      'poiList': poiList?.map((p) => p.toMap()).toList(),
      'sematicDescription': sematicDescription
    };
  }
}

/// 地址结果的层次化信息
class TIMAddressComponent implements TIMLocationBaseModel {
  /// 国家
  String? country;

  /// 省份名称
  String? province;

  /// 城市名称
  String? city;

  /// 区县名称
  String? district;

  /// 乡镇
  String? town;

  /// 有参构造
  TIMAddressComponent({
    this.country,
    this.province,
    this.city,
    this.district,
    this.town,
  });

  /// map => TIMAddressComponent
  TIMAddressComponent.fromMap(Map map)
      : assert(
  map != null, // ignore: unnecessary_null_comparison
  'Construct a TIMAddressComponent，The parameter map cannot be null !') {
    country = map['country'];
    province = map['province'];
    city = map['city'];
    district = map['district'];
    town = map['town'];
  }
  @override
  fromMap(Map map) {
    return TIMAddressComponent.fromMap(map);
  }

  @override
  Map<String, Object?> toMap() {
    return {
      'country': country,
      'province': province,
      'city': city,
      'district': district,
      'town': town,
    };
  }
}

/// 枚举：地图区域改变原因
enum TIMRegionChangeReason {
  ///<手势触发导致地图区域变化，如双击、拖拽、滑动地图
  Gesture,

  ///<地图上控件事件，如点击指南针返回2D地图。
  Event,

  ///<开发者调用接口、设置地图参数等导致地图区域变化
  APIs,
}
